.table > thead > tr > th {
    vertical-align: middle;
}

.table > tbody > tr > td {
    vertical-align: middle;
}

.navbar {
    padding: 0.75rem 1.0rem !important;
}

.card-header {
    padding: 0.3rem 1.0rem !important;
}

.card-footer {
    padding: 0.3rem 1.0rem !important;
}

.ara-label {
    font-size: 90% !important;
    padding: 0.10rem 0.25rem !important;
    margin-bottom: 3px;
}

.ara-result-status-badge {
    font-size: 90% !important;
}

.custom-checkbox .custom-control-input:checked ~ .custom-control-label::before {
  background-color: #b4bcc2;
}

.custom-checkbox .custom-control-input:active ~ .custom-control-label::before {
  background-color: #ffffff;
}
