# Generated by Django 2.2.1 on 2019-05-23 17:34

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('api', '0001_initial'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='host',
            name='alias',
        ),
    ]
