# Generated by Django 2.2.16 on 2020-09-06 18:51

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('api', '0005_unique_label_names'),
    ]

    # Previously, choices included "ignored" and "changed" but these were never used
    # See: https://github.com/ansible-community/ara/issues/150
    operations = [
        migrations.AlterField(
            model_name='result',
            name='status',
            field=models.CharField(choices=[('ok', 'ok'), ('failed', 'failed'), ('skipped', 'skipped'), ('unreachable', 'unreachable'), ('unknown', 'unknown')], default='unknown', max_length=25),
        ),
    ]
