# Generated by Django 2.2.17 on 2020-12-04 04:38

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('api', '0007_add_expired_status'),
    ]

    operations = [
        migrations.AddField(
            model_name='playbook',
            name='controller',
            field=models.CharField(default='localhost', max_length=255),
        ),
    ]
