<div
  class="modal fade"
  id="aboutModal"
  tabindex="-1"
  role="dialog"
  aria-labelledby="aboutModalLabel"
  aria-hidden="true"
>
  <div class="modal-dialog" role="document">
    <div class="modal-content">
      <div class="modal-header">
        <h5 class="modal-title" id="aboutModalLabel">ara {{ ARA_VERSION }}</h5>
        <button
          type="button"
          class="close"
          data-dismiss="modal"
          aria-label="Close"
        >
          <span aria-hidden="true">&times;</span>
        </button>
      </div>
      <ul class="list-group list-group-flush">
        <li class="list-group-item">
          <p>ARA Records Ansible and makes it easier to understand and troubleshoot. It is another recursive acronym.</p>
          <p>ara is a free and open source project under the GPLv3 license.</p>
        </li>
        <li class="list-group-item">Code: <a href="https://github.com/ansible-community/ara" target="_blank">https://github.com/ansible-community/ara</a></li>
        <li class="list-group-item">Website: <a href="https://ara.recordsansible.org" target="_blank">https://ara.recordsansible.org</a></li>
      </ul>
      <div class="modal-footer">
        <button type="button" class="btn btn-secondary" data-dismiss="modal">
          Close
        </button>
      </div>
    </div>
  </div>
</div>
