{% extends "base.html" %}
{% load pygments_highlights %}
{% block body %}
  <div>
    {% include "partials/playbook_card.html" with playbook=record.playbook %}
  </div>
  <br/>
  <div>
  <div class="card">
    <div class="card-header">
      <h4 title="key saved with ara_record">Record: {{ record.key }}</h4>
    </div>
    <div class="card-body">
      {{ record.value | format_data | safe }}
    </div>
  </div>
{% endblock %}
