# Generated by Django 2.2.1 on 2019-05-30 16:00

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('api', '0002_remove_host_alias'),
    ]

    operations = [
        migrations.AddField(
            model_name='result',
            name='changed',
            field=models.BooleanField(default=False),
        ),
        migrations.AddField(
            model_name='result',
            name='ignore_errors',
            field=models.BooleanField(default=False),
        ),
    ]
